import { Request, Response } from "express";
import bcrypt from "bcrypt";
import { User } from "../models/user";
import { saveSession } from "../utils/utils";
import { MONGODB_IGNORED_FIELDS } from "../utils/constants";

export async function SignUp(req: Request, res: Response) {
  if (!req.body.accountName || !req.body.username || !req.body.password) {
    return res.status(400).json({ status: "err" });
  }

  const { accountName, username, password } = req.body;

  const existingUser = await User.findOne({ accountName })
    .select("accountName -_id")
    .lean();

  if (existingUser) {
    return res.status(400).json({ status: 1 });
  }

  const isBase64Password =
    /^([A-Za-z0-9+/]{4})*([A-Za-z0-9+/]{4}|[A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{2}==)$/.test(
      password
    );

  if (!isBase64Password) {
    return res.status(400).json({ status: "err" });
  }

  const decodedPassword = Buffer.from(password, "base64").toString("utf-8");

  const hashedPassword = await bcrypt.hash(decodedPassword, 10);

  const user = new User({
    accountName: accountName,
    username: username,
    password: hashedPassword,
  });

  user
    .save()
    .then(() => saveSession(res, accountName))
    .catch((err) => {
      console.log(err);
      res.status(500).json({ status: "err" });
    });
}

export async function Login(req: Request, res: Response) {
  try {
    if (!req.body.accountName || !req.body.password) {
      return res.status(400).json({ status: "err" });
    }

    const { accountName, password } = req.body;

    const user = await User.findOne({ accountName }).lean();

    if (!user) {
      return res.status(401).json({ status: "err" });
    }

    if (
      user.accountName === null ||
      user.accountName === undefined ||
      user.password === undefined ||
      user.password === null
    ) {
      return res.status(401).json({ status: "err" });
    }

    const decodedPassword = Buffer.from(password, "base64").toString("utf-8");

    const isPasswordValid = await bcrypt.compare(
      decodedPassword,
      user.password
    );

    if (!isPasswordValid) {
      return res.status(401).json({ status: "err" });
    }

    saveSession(res, user.accountName);
  } catch (error) {
    console.error("error on login:", error);
    res.status(500).json({ status: "err" });
  }
}

export async function GetUserProfile(req: Request, res: Response) {
  try {
    const user = await User.findOne({
      accountName: req.params.accountName,
    })
      .select(MONGODB_IGNORED_FIELDS)
      .lean();

    if (!user) {
      return res.status(404).json({ status: "err" });
    }

    console.log("user:", user);

    res.json({
      accountName: user.accountName,
      username: user.username,
    });
  } catch (error) {
    console.error("error on get user profile:", error);
    res.status(500).json({ status: "err" });
  }
}
